@@ -20,6 +20,9 @@
  * PERFORMANCE OF THIS SOFTWARE.
  */
 
+#ifndef AST_2DTOOL_H
+#define AST_2DTOOL_H
+
 /* Eng Capability Definition */
 #define	ENG_CAP_Sync				0x0001
 #define	ENG_CAP_ScreenToScreenCopy		0x0002
@@ -621,3 +624,7 @@ typedef struct {
         addr->PKT_SC_dwHeader  = (ULONG)(PKT_SINGLE_CMD_HEADER + CMDQREG_LINE_NUMBER); 	\
         addr->PKT_SC_dwData[0] = (ULONG)(no);					\
       }
+
+Bool bASTGetLineTerm(_LINEInfo *LineInfo, LINEPARAM *dsLineParam);
+
+#endif /* AST_2DTOOL_H */
