@@ -298,6 +298,7 @@ ASTProbe(DriverPtr drv, int flags)
 
 	        foundScreen = TRUE;
 
+		free(pEnt);
 	    } /* end of if */
         }  /* end of for-loop */
     } /* end of if flags */
@@ -1207,8 +1208,8 @@ ASTSaveScreen(ScreenPtr pScreen, Bool unblack)
    if ((pScrn != NULL) && pScrn->vtSema) {
      ASTBlankScreen(pScrn, unblack);
    }
-   return (TRUE);
 #endif
+   return (TRUE);
 }
 
 static Bool
@@ -1484,9 +1485,9 @@ ASTDoDDC(ScrnInfoPtr pScrn, int index)
 
    xf86MonPtr MonInfo1 = NULL, MonInfo2 = NULL;
    unsigned long i, j, k;
-   struct monitor_ranges ranges, ranges1, ranges2;
+   struct monitor_ranges ranges, ranges1 = {0}, ranges2 = {0};
    int DTSelect, dclock1=0, h_active1=0, v_active1=0, dclock2=0, h_active2=0, v_active2=0;
-   struct std_timings stdtiming, *stdtiming1, *stdtiming2;
+   struct std_timings stdtiming, *stdtiming1 = {0}, *stdtiming2 = {0};
 
    /* Honour Option "noDDC" */
    if (xf86ReturnOptValBool(pAST->Options, OPTION_NO_DDC, FALSE)) {
@@ -1797,7 +1798,6 @@ static XF86AttributeRec ASTAttributes[NUM_ATTRIBUTES] =
 static void ASTStopVideo(ScrnInfoPtr pScrn, pointer data, Bool exit)
 {
     ASTPortPrivPtr pPriv = (ASTPortPrivPtr)data;
-    ASTPtr pAST = ASTPTR(pScrn);
 
     REGION_EMPTY(pScrn->pScreen, &pPriv->clip);
 
