@@ -20,6 +20,9 @@
  * PERFORMANCE OF THIS SOFTWARE.
  */
 
+#ifndef AST_MODE_H
+#define AST_MODE_H
+
 /* Mode Limitation */
 #define MAX_HResolution		1920
 #define MAX_VResolution		1200
@@ -85,7 +88,7 @@
 #define DAC_NUM_EGA		64
 #define DAC_NUM_VGA		256
 
-/* Data Structure decalration for internal use */
+/* Data Structure declaration for internal use */
 typedef struct {
 
     UCHAR MISC;
@@ -132,3 +135,5 @@ typedef struct {
     PVBIOS_ENHTABLE_STRUCT pEnhTableEntry;
 
 } VBIOS_MODE_INFO, *PVBIOS_MODE_INFO;
+
+#endif /* AST_MODE_H */
